@@ -8,6 +8,12 @@
 // Copyright (c) 2022 Tomas Protivinsky, All rights reserved.
 //      https://github.com/protivinsky/pytdigest
 
+#ifdef _MSC_VER
+#define DLL_EXPORT __declspec(dllexport)
+#else
+#define DLL_EXPORT
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdbool.h>
@@ -18,40 +24,40 @@ typedef struct tdigest tdigest_t;
 
 // td_new allocates a new histogram.
 // It is similar to init but assumes that it can use malloc.
-tdigest_t *td_new(double compression);
+DLL_EXPORT tdigest_t *td_new(double compression);
 
 // td_free frees the memory associated with h.
-void td_free(tdigest_t *h);
+DLL_EXPORT void td_free(tdigest_t *h);
 
 // td_add adds val to h with the specified count.
-void td_add(tdigest_t *h, double val, double count);
+DLL_EXPORT void td_add(tdigest_t *h, double val, double count);
 
 // td_merge merges the data from from into into.
-void td_merge(tdigest_t *into, tdigest_t *from);
+DLL_EXPORT void td_merge(tdigest_t *into, tdigest_t *from);
 
 // td_reset resets a histogram.
 void td_reset(tdigest_t *h);
 
 // td_value_at queries h for the value at q.
 // If q is not in [0, 1], NAN will be returned.
-double td_value_at(tdigest_t *h, double q);
+DLL_EXPORT double td_value_at(tdigest_t *h, double q);
 
 // td_value_at queries h for the quantile of val.
 // The returned value will be in [0, 1].
-double td_quantile_of(tdigest_t *h, double val);
+DLL_EXPORT double td_quantile_of(tdigest_t *h, double val);
 
 // td_trimmed_mean returns the mean of data from the lo quantile to the
 // hi quantile.
 double td_trimmed_mean(tdigest_t *h, double lo, double hi);
 
 // td_total_count returns the total count contained in h.
-double td_total_weight(tdigest_t *h);
+DLL_EXPORT double td_total_weight(tdigest_t *h);
 
 // td_total_sum returns the sum of all the data added to h.
-double td_total_sum(tdigest_t *h);
+DLL_EXPORT double td_total_sum(tdigest_t *h);
 
 // td_scale_weight multiplies all counts by factor.
-void td_scale_weight(tdigest_t *h, double factor);
+DLL_EXPORT void td_scale_weight(tdigest_t *h, double factor);
 
 // td_shift shifts the whole distribution (add shift to all means)
 void td_shift(tdigest_t *h, double shift);
@@ -124,7 +130,7 @@ static int next_centroid(tdigest_t *td) {
     return td->num_merged + td->num_unmerged;
 }
 
-void merge(tdigest_t *td);
+DLL_EXPORT void merge(tdigest_t *td);
 
 
 /*** CONSTRUCTORS ***/
@@ -141,7 +147,7 @@ static tdigest_t *td_init(double delta, size_t buf_size, char *buf) {
     if (!td) {
         return NULL;
     }
-    bzero((void *) (td), buf_size);
+    memset((void *) (td), 0, buf_size);
     *td = (tdigest_t) {
             .delta = delta,
             .max_centroids = (buf_size - sizeof(tdigest_t)) / sizeof(centroid_t),
@@ -153,16 +159,16 @@ static tdigest_t *td_init(double delta, size_t buf_size, char *buf) {
     return td;
 }
 
-tdigest_t *td_new(double delta) {
+DLL_EXPORT tdigest_t *td_new(double delta) {
     size_t memsize = required_buffer_size(delta);
     return td_init(delta, memsize, (char *) (malloc(memsize)));
 }
 
-void td_free(tdigest_t *td) {
+DLL_EXPORT void td_free(tdigest_t *td) {
     free((void *) (td));
 }
 
-void td_merge(tdigest_t *into, tdigest_t *from) {
+DLL_EXPORT void td_merge(tdigest_t *into, tdigest_t *from) {
     merge(into);
     merge(from);
     for (int i = 0; i < from->num_merged; i++) {
@@ -172,15 +178,14 @@ void td_merge(tdigest_t *into, tdigest_t *from) {
 }
 
 void td_reset(tdigest_t *td) {
-    bzero((void *) (&td->centroids[0]), sizeof(centroid_t) * td->max_centroids);
+    memset((void *) (&td->centroids[0]), 0, sizeof(centroid_t) * td->max_centroids);
     td->num_merged = 0;
     td->merged_weight = 0;
     td->num_unmerged = 0;
     td->unmerged_weight = 0;
 }
 
-// do I need this? or can I just remove it?
-void td_scale_weight(tdigest_t *td, double factor) {
+DLL_EXPORT void td_scale_weight(tdigest_t *td, double factor) {
     merge(td);
     td->merged_weight *= factor;
     td->unmerged_weight *= factor; // this should be unnecessary, right?
@@ -196,11 +201,11 @@ void td_shift(tdigest_t *td, double shift) {
     }
 }
 
-double td_total_weight(tdigest_t *td) {
+DLL_EXPORT double td_total_weight(tdigest_t *td) {
     return td->merged_weight + td->unmerged_weight;
 }
 
-double td_total_sum(tdigest_t *td) {
+DLL_EXPORT double td_total_sum(tdigest_t *td) {
     centroid_t *c = NULL;
     double sum = 0;
     int num_centroids = td->num_merged + td->num_unmerged;
@@ -211,7 +216,7 @@ double td_total_sum(tdigest_t *td) {
     return sum;
 }
 
-void td_add(tdigest_t *td, double mean, double weight) {
+DLL_EXPORT void td_add(tdigest_t *td, double mean, double weight) {
     if (should_merge(td)) {
         merge(td);
     }
@@ -223,7 +228,7 @@ void td_add(tdigest_t *td, double mean, double weight) {
     td->unmerged_weight += weight;
 }
 
-void merge(tdigest_t *td) {
+DLL_EXPORT void merge(tdigest_t *td) {
     if (td->num_unmerged == 0) {
         return;
     }
@@ -263,7 +268,7 @@ void merge(tdigest_t *td) {
 }
 
 // this is CDF, right?
-double td_quantile_of(tdigest_t *td, double val) {
+DLL_EXPORT double td_quantile_of(tdigest_t *td, double val) {
     merge(td);
     if (td->num_merged == 0) {
         return NAN;
@@ -302,7 +307,7 @@ double td_quantile_of(tdigest_t *td, double val) {
 }
 
 // this is inverse cdf, right? or quantile, that's the same
-double td_value_at(tdigest_t *td, double q) {
+DLL_EXPORT double td_value_at(tdigest_t *td, double q) {
     merge(td);
     if (q < 0 || q > 1 || td->num_merged == 0) {
         return NAN;
@@ -383,7 +388,7 @@ double td_trimmed_mean(tdigest_t *td, double lo, double hi) {
 }
 
 
-void td_add_batch(tdigest_t *td, int num_values, double *means, double *weights) {
+DLL_EXPORT void td_add_batch(tdigest_t *td, int num_values, double *means, double *weights) {
     for (int i = 0; i < num_values; i++) {
         if (should_merge(td)) {
             merge(td);
@@ -398,14 +403,14 @@ void td_add_batch(tdigest_t *td, int num_values, double *means, double *weights)
 }
 
 
-void td_cdf_batch(tdigest_t *td, int count, const double *values, double *quantiles) {
+DLL_EXPORT void td_cdf_batch(tdigest_t *td, int count, const double *values, double *quantiles) {
     for (int i = 0; i < count; i++) {
         quantiles[i] = td_quantile_of(td, values[i]);
     }
 }
 
 
-void td_inverse_cdf_batch(tdigest_t *td, int count, const double *quantiles, double *values) {
+DLL_EXPORT void td_inverse_cdf_batch(tdigest_t *td, int count, const double *quantiles, double *values) {
     for (int i = 0; i < count; i++) {
         values[i] = td_value_at(td, quantiles[i]);
     }
@@ -426,21 +431,21 @@ void td_inverse_cdf_batch(tdigest_t *td, int count, const double *quantiles, dou
 //    return other;
 //}
 
-centroid_t *td_get_centroid(tdigest_t *td, int i) {
+DLL_EXPORT centroid_t *td_get_centroid(tdigest_t *td, int i) {
     if (i >= td->num_merged + td->num_unmerged) {
         return NULL;
     }
     return &td->centroids[i];
 }
 
-void td_get_centroids(tdigest_t *td, double *centroids) {
+DLL_EXPORT void td_get_centroids(tdigest_t *td, double *centroids) {
     for (int i = 0; i < td->num_merged + td->num_unmerged; i++) {
         centroids[2 * i] = td->centroids[i].mean;
         centroids[2 * i + 1] = td->centroids[i].weight;
     }
 }
 
-tdigest_t *td_of_centroids(double delta, int num_centroids, double *centroids) {
+DLL_EXPORT tdigest_t *td_of_centroids(double delta, int num_centroids, double *centroids) {
     tdigest_t *td = td_new(delta);
     double total_weight = 0;
     for (int i = 0; i < num_centroids; i++) {
@@ -456,7 +461,7 @@ tdigest_t *td_of_centroids(double delta, int num_centroids, double *centroids) {
     return td;
 }
 
-void td_fill_centroids(tdigest_t *td, int num_centroids, double *centroids) {
+DLL_EXPORT void td_fill_centroids(tdigest_t *td, int num_centroids, double *centroids) {
     double total_weight = 0;
     int cap = (num_centroids < td->max_centroids) ? num_centroids : td->max_centroids;
     for (int i = 0; i < cap; i++) {
@@ -472,9 +477,7 @@ void td_fill_centroids(tdigest_t *td, int num_centroids, double *centroids) {
 }
 
 
-
 //int main() {
-//    printf("Hello world.\n");
 //
 //    tdigest_t *td = td_new(100.);
 //    td_add(td, 10., 1.);
